mod migration;
mod migrations;
mod program;
mod section;
mod sql_json;

pub use migration::*;
pub use migrations::create_migrations;
pub use program::*;

pub use rusqlite::Connection as DbConn;

use sprinklers_core::model::Sections;

use eyre::Result;
use rusqlite::NO_PARAMS;

pub fn setup_db() -> Result<DbConn> {
    // let conn = DbConn::open_in_memory()?;
    let mut conn = DbConn::open("test.db")?;

    // Go ahead and use write ahead log for better perf
    conn.execute_batch("PRAGMA journal_mode=WAL;")?;

    let migs = create_migrations();
    migs.apply(&mut conn)?;

    Ok(conn)
}

pub fn query_sections(conn: &DbConn) -> Result<Sections> {
    let mut statement = conn.prepare_cached(
        "SELECT s.id, s.name, s.interface_id \
        FROM sections AS s;",
    )?;
    let rows = statement.query_map(NO_PARAMS, section::from_sql)?;
    let mut sections = Sections::new();
    for row in rows {
        let section = row?;
        sections.insert(section.id, section.into());
    }
    Ok(sections)
}
